"use strict";
require('reflect-metadata');
var httpStatus = require('http-status');
/**
 * Default error handler. Does console.error with the error and sets the
 * http response code to 500.
 *
 * @param {Request} req - express request object
 * @param {Response} res - express response object
 * @param {Error} err - error that happend
 */
exports.DEFAULT_ERROR_HANDLER = function (req, res, err) {
    console.error(err);
    res.status(httpStatus.INTERNAL_SERVER_ERROR).end();
};
/**
 * Manager object that handles all ErrorHandlerFunctions for a controller. Registers itself in the controllers
 * metadata. If an error is thrown, just pass it to the error handler. The handler will do all the magic.
 *
 * @class
 */
var ControllerErrorHandler = (function () {
    function ControllerErrorHandler() {
        this.handlers = { 'Error': exports.DEFAULT_ERROR_HANDLER };
    }
    /**
     * Adds an error handler for the current controller with the given errorType. If the errorType is omitted,
     * the handler registers the function as a 'default' error handler.
     *
     * @param {ErrorHandlerFunction} handler - The error handler to register.
     * @param {Error} [errorType] - Error class that should be registered. If omitted, the handler is registered as default.
     */
    ControllerErrorHandler.prototype.addHandler = function (handler, errorType) {
        var type = (errorType || Error).name;
        if (this.handlers[type] && this.handlers[type] !== exports.DEFAULT_ERROR_HANDLER) {
            console.warn("Duplicate error handler declaration for type '" + type + "'.");
        }
        this.handlers[type] = handler;
    };
    /**
     * Calls the error handler for a given error.
     *
     * @param {any} context - Context of the error handler.
     * @param {Request} request - ExpressJS request object.
     * @param {Response} response - ExpressJS response object.
     * @param {Error} error - Error object that was thrown.
     */
    ControllerErrorHandler.prototype.handleError = function (context, request, response, error) {
        if (!(error instanceof Error)) {
            error = new Error(error);
        }
        var proto = Object.getPrototypeOf(error);
        while (proto) {
            var type = proto.constructor;
            if (this.handlers[type.name]) {
                this.handlers[type.name].apply(context, [request, response, error]);
                break;
            }
            proto = Object.getPrototypeOf(proto);
        }
    };
    return ControllerErrorHandler;
}());
exports.ControllerErrorHandler = ControllerErrorHandler;
