"use strict";
require('reflect-metadata');
var ControllerErrorHandler_1 = require('./ControllerErrorHandler');
var Errors_1 = require('./Errors');
var ARGUMENT_COUNT = 3;
/**
 * Reflect metadata key for error handler manager.
 * @type {string}
 */
exports.ERRORHANDLER_KEY = 'errorHandler';
/**
 * Errorhandler decorator; decorates the given function as an error handler for the current controller.
 * You can specify various errors in the open parameter list. The handler is registered for the given error types,
 * and is only called if the thrown error matches the registered type. If errors are omitted, the handler will be
 * registered as 'default' and is called if no other specialized handler matches the thrown error.
 *
 * @param {...Error[]} errors - List of error classes to register to.
 * @returns {(any, string, PropertyDescriptor) => void} - Decorator for the class function.
 */
function ErrorHandler() {
    var errors = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        errors[_i - 0] = arguments[_i];
    }
    return function (target, propertyKey, descriptor) {
        var paramtypes = Reflect.getMetadata('design:paramtypes', target, propertyKey);
        if (paramtypes.length !== ARGUMENT_COUNT) {
            throw new Errors_1.ErrorHandlerWrongArgumentsError();
        }
        [Object, Object].forEach(function (type, index) {
            if (paramtypes[index] !== type) {
                throw new Errors_1.ErrorHandlerWrongArgumentTypesError();
            }
        });
        var returnValue = Reflect.getMetadata('design:returntype', target, propertyKey);
        if (!!returnValue) {
            throw new Errors_1.ErrorHandlerWrongReturnTypeError();
        }
        var handler = Reflect.getMetadata(exports.ERRORHANDLER_KEY, target.constructor) || new ControllerErrorHandler_1.ControllerErrorHandler();
        if (errors.length) {
            errors.forEach(function (e) { return handler.addHandler(descriptor.value, e); });
        }
        else {
            handler.addHandler(descriptor.value);
        }
        Reflect.defineMetadata(exports.ERRORHANDLER_KEY, handler, target.constructor);
    };
}
exports.ErrorHandler = ErrorHandler;
