/// <reference types="node" />
import { RouteMethod } from '../routes/RouteDecorators';
export declare class DesigntimeError extends Error {
    constructor();
}
export declare class RuntimeError extends Error {
    constructor();
}
export declare class ParameterError extends RuntimeError {
    constructor();
}
export declare class RouteError extends RuntimeError {
    constructor();
}
export declare class ValidationError extends ParameterError {
    constructor();
}
/**
 * Error that is thrown when a http method is not supported by the framework.
 * (Error will happen at startup)
 *
 * @class HttpVerbNotSupportedError
 * @extends {DesigntimeError}
 */
export declare class HttpVerbNotSupportedError extends DesigntimeError {
    constructor(method: RouteMethod);
}
/**
 * Error that is thrown when a route is duplicated (same url and method).
 * (Error will happen at startup)
 *
 * @class DuplicateRouteDeclarationError
 * @extends {DesigntimeError}
 */
export declare class DuplicateRouteDeclarationError extends DesigntimeError {
    constructor(url: string, method: RouteMethod);
}
/**
 * Error that is thrown when a head route has a non boolean return type.
 * (Error will happen at startup)
 *
 * @class HeadHasWrongReturnTypeError
 * @extends {DesigntimeError}
 */
export declare class HeadHasWrongReturnTypeError extends DesigntimeError {
    constructor();
}
/**
 * Error that is thrown when an error handler accepts the wrong arguments.
 * (Error will happen at startup)
 *
 * @class ErrorHandlerWrongArgumentsError
 * @extends {DesigntimeError}
 */
export declare class ErrorHandlerWrongArgumentsError extends DesigntimeError {
    constructor();
}
/**
 * Error that is thrown when an error handler accepts the wrong argument types.
 * (Error will happen at startup)
 *
 * @class ErrorHandlerWrongArgumentTypesError
 * @extends {DesigntimeError}
 */
export declare class ErrorHandlerWrongArgumentTypesError extends DesigntimeError {
    constructor();
}
/**
 * Error that is thrown when an error handler has a non void return type.
 * (Error will happen at startup)
 *
 * @class ErrorHandlerWrongReturnTypeError
 * @extends {DesigntimeError}
 */
export declare class ErrorHandlerWrongReturnTypeError extends DesigntimeError {
    constructor();
}
/**
 * Error that is thrown when a given parameter type supports not at least argument (used for instantiation of the type).
 * (Error will happen at startup)
 *
 * @class ParameterParseError
 * @extends {DesigntimeError}
 */
export declare class ParameterConstructorArgumentsError extends DesigntimeError {
    constructor(name: string);
}
/**
 * Error that is thrown when a required parameter is not provided. Url parameters are always required.
 * (Runtime error)
 *
 * @class RequiredParameterNotProvidedError
 * @extends {ParameterError}
 */
export declare class RequiredParameterNotProvidedError extends ParameterError {
    constructor(name: string);
}
/**
 * Error that is thrown when the parsing process of a parameters throws an error.
 * (Runtime error)
 *
 * @class ParameterParseError
 * @extends {ParameterError}
 */
export declare class ParameterParseError extends ParameterError {
    innerException: Error;
    constructor(name: string, innerException: Error);
}
/**
 * Error that is thrown when the route method returns the wrong type (i.e. a string instead of a number).
 * (Runtime error)

 * @class WrongReturnTypeError
 * @extends {RouteError}
 */
export declare class WrongReturnTypeError extends RouteError {
    constructor(name: string, expected: Function, received: Function);
}
/**
 * Error that is thrown when the route method throws any error (or any Promise returned by a route).
 * (Runtime error)
 *
 * @class GenericRouteError
 * @extends {RouteError}
 */
export declare class GenericRouteError extends RouteError {
    innerException: Error;
    constructor(name: string, innerException: Error);
}
/**
 * Error that is thrown when the - if provided - validator of a parameter returns "false".
 * (Runtime error)
 *
 * @class ParamValidationFailedError
 * @extends {ValidationError}
 */
export declare class ParamValidationFailedError extends ValidationError {
    constructor(name: string);
}
/**
 * Error that is thrown when a version information on a route or a controller is missing both information,
 * i.e. there is no version in "from" and no version in "until"
 * (Error will happen at startup)
 *
 * @class VersionInformationMissing
 * @extends {DesigntimeError}
 */
export declare class VersionInformationMissing extends DesigntimeError {
    constructor(ctrlOrRouteName: string);
}
/**
 * Error that is thrown when a version information contains errors.
 * (Error will happen at startup)
 *
 * @class VersionInformationInvalid
 * @extends {DesigntimeError}
 */
export declare class VersionInformationInvalid extends DesigntimeError {
    constructor(ctrlOrRouteName: string, reason: string);
}
/**
 * Error that is thrown when a route or a controller contains multiple
 * version information.
 * (Error will happen at startup)
 *
 * @class DuplicateVersionInformation
 * @extends {DesigntimeError}
 */
export declare class DuplicateVersionInformation extends DesigntimeError {
    constructor(ctrlOrRouteName: string);
}
