/// <reference types="chai" />
import 'reflect-metadata';
import { BodyParamOptions, CookieParamOptions, ParamOptions, QueryParamOptions } from './ParamOptions';
/**
 * Reflect metadata key for parameter list.
 * @type {string}
 */
export declare const PARAMS_KEY: string;
/**
 * Enum for parameter type.
 */
export declare enum ParamType {
    Url = 0,
    Query = 1,
    Body = 2,
    Request = 3,
    Response = 4,
    Header = 5,
    Cookie = 6,
}
/**
 * Declares the current parameter as a query parameter. The route will provide this value from the request.query object.
 *
 * @param {string} name - the name of the parameter (inside the query object)
 * @param {QueryParamOptions} options - The specific options for this parameter.
 * @returns {(Object, string, number) => void} - Parameter decorator for the given function.
 */
export declare function Query(name: string, options?: QueryParamOptions): (target: Object, propertyKey: string, parameterIndex: number) => void;
/**
 * Declares the current parameter as an url parameter. The route will provide this value from the request.params object.
 *
 * @param {string} name - the name of the parameter (inside the params object)
 * @returns {(Object, string, number) => void} - Parameter decorator for the given function.
 */
export declare function UrlParam(name: string): (target: Object, propertyKey: string, parameterIndex: number) => void;
/**
 * Declares the current parameter as a body parameter. The route will provide the request.body value.
 *
 * @param {ParamOptions} options - The specific options for this parameter.
 * @returns {(Object, string, number) => void} - Parameter decorator for the given function.
 */
export declare function Body(options?: BodyParamOptions): (target: Object, propertyKey: string, parameterIndex: number) => void;
/**
 * Declares the current parameter as a request parameter. The route will provide the express js request object.
 *
 * @returns {(Object, string, number) => void} - Parameter decorator for the given function.
 */
export declare function Req(): (target: Object, propertyKey: string, parameterIndex: number) => void;
/**
 * Declares the current parameter as a response parameter. The route will provide the express js response object.
 *
 * @returns {(Object, string, number) => void} - Parameter decorator for the given function.
 */
export declare function Res(): (target: Object, propertyKey: string, parameterIndex: number) => void;
/**
 * Declares the current parameter as a header parameter. The route will provide the corresponding http header value.
 *
 * @returns {(Object, string, number) => void} - Parameter decorator for the given function.
 */
export declare function Header(name: string, options?: ParamOptions): (target: Object, propertyKey: string, parameterIndex: number) => void;
/**
 * Declares the current parameter as a cookie parameter. The route will provide the corresponding cookie value.
 *
 * @returns {(Object, string, number) => void} - Parameter decorator for the given function.
 */
export declare function Cookie(name: string, options?: CookieParamOptions): (target: Object, propertyKey: string, parameterIndex: number) => void;
