"use strict";
require('reflect-metadata');
var ParamRegistration_1 = require('../models/ParamRegistration');
/**
 * Reflect metadata key for parameter list.
 * @type {string}
 */
exports.PARAMS_KEY = 'params';
/**
 * Enum for parameter type.
 */
(function (ParamType) {
    ParamType[ParamType["Url"] = 0] = "Url";
    ParamType[ParamType["Query"] = 1] = "Query";
    ParamType[ParamType["Body"] = 2] = "Body";
    ParamType[ParamType["Request"] = 3] = "Request";
    ParamType[ParamType["Response"] = 4] = "Response";
    ParamType[ParamType["Header"] = 5] = "Header";
    ParamType[ParamType["Cookie"] = 6] = "Cookie";
})(exports.ParamType || (exports.ParamType = {}));
var ParamType = exports.ParamType;
function param(type, name, options) {
    return function (target, propertyKey, parameterIndex) {
        var paramtypes = Reflect.getMetadata('design:paramtypes', target, propertyKey);
        var params = Reflect.getOwnMetadata(exports.PARAMS_KEY, target, propertyKey) || [];
        params.push(new ParamRegistration_1.ParamRegistration(type, name, paramtypes[parameterIndex], parameterIndex, options));
        Reflect.defineMetadata(exports.PARAMS_KEY, params, target, propertyKey);
    };
}
/**
 * Declares the current parameter as a query parameter. The route will provide this value from the request.query object.
 *
 * @param {string} name - the name of the parameter (inside the query object)
 * @param {QueryParamOptions} options - The specific options for this parameter.
 * @returns {(Object, string, number) => void} - Parameter decorator for the given function.
 */
function Query(name, options) {
    return param(ParamType.Query, name, options);
}
exports.Query = Query;
/**
 * Declares the current parameter as an url parameter. The route will provide this value from the request.params object.
 *
 * @param {string} name - the name of the parameter (inside the params object)
 * @returns {(Object, string, number) => void} - Parameter decorator for the given function.
 */
function UrlParam(name) {
    return param(ParamType.Url, name, { required: true });
}
exports.UrlParam = UrlParam;
/**
 * Declares the current parameter as a body parameter. The route will provide the request.body value.
 *
 * @param {ParamOptions} options - The specific options for this parameter.
 * @returns {(Object, string, number) => void} - Parameter decorator for the given function.
 */
function Body(options) {
    return param(ParamType.Body, 'body', options);
}
exports.Body = Body;
/**
 * Declares the current parameter as a request parameter. The route will provide the express js request object.
 *
 * @returns {(Object, string, number) => void} - Parameter decorator for the given function.
 */
function Req() {
    return param(ParamType.Request, 'request');
}
exports.Req = Req;
/**
 * Declares the current parameter as a response parameter. The route will provide the express js response object.
 *
 * @returns {(Object, string, number) => void} - Parameter decorator for the given function.
 */
function Res() {
    return param(ParamType.Response, 'response');
}
exports.Res = Res;
/**
 * Declares the current parameter as a header parameter. The route will provide the corresponding http header value.
 *
 * @returns {(Object, string, number) => void} - Parameter decorator for the given function.
 */
function Header(name, options) {
    return param(ParamType.Header, name, options);
}
exports.Header = Header;
/**
 * Declares the current parameter as a cookie parameter. The route will provide the corresponding cookie value.
 *
 * @returns {(Object, string, number) => void} - Parameter decorator for the given function.
 */
function Cookie(name, options) {
    return param(ParamType.Cookie, name, options);
}
exports.Cookie = Cookie;
