"use strict";
require('reflect-metadata');
var RouteRegistration_1 = require('../models/RouteRegistration');
/**
 * Reflect metadata key for the controllers routes.
 *
 * @type {string}
 */
exports.ROUTES_KEY = 'routes';
/**
 * Enum for the possible route http methods.
 */
(function (RouteMethod) {
    RouteMethod[RouteMethod["Get"] = 0] = "Get";
    RouteMethod[RouteMethod["Put"] = 1] = "Put";
    RouteMethod[RouteMethod["Post"] = 2] = "Post";
    RouteMethod[RouteMethod["Delete"] = 3] = "Delete";
    RouteMethod[RouteMethod["Head"] = 4] = "Head";
})(exports.RouteMethod || (exports.RouteMethod = {}));
var RouteMethod = exports.RouteMethod;
/**
 * Declares the given method as an api route. Adds the route registration to the controller.
 * All route registrations are decorated and registered during the registerControllers method.
 *
 * @param {string} [route=''] - The routed endpoint of the method. If omitted, the base route is taken.
 * @param {RouteMethod} httpMethod - The http verb to use for this route.
 * @param {RequestHandler[]} [middlewares=[]] - The used middleware(s) for this route.
 * @returns {(any, string, PropertyDescriptor) => void} - Method decorator for the given function.
 */
function Route(route, httpMethod) {
    if (route === void 0) { route = ''; }
    if (httpMethod === void 0) { httpMethod = RouteMethod.Get; }
    var middlewares = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        middlewares[_i - 2] = arguments[_i];
    }
    return function (target, propertyKey, descriptor) {
        var routes = Reflect.getMetadata(exports.ROUTES_KEY, target.constructor) || [];
        routes.push(new RouteRegistration_1.RouteRegistration(route || '', httpMethod, descriptor, propertyKey, middlewares));
        Reflect.defineMetadata(exports.ROUTES_KEY, routes, target.constructor);
    };
}
exports.Route = Route;
/**
 * Alias function for @Route(string, RouteMethod.Get)
 *
 * @param {string} [route=''] - The routed endpoint of the method. If omitted, the base route is taken.
 * @param {RequestHandler[]} [middlewares=[]] - The used middleware(s) for this route.
 * @returns {(any, string, PropertyDescriptor) => void} - Method decorator for the given function with the http verb 'GET'.
 */
function Get(route) {
    if (route === void 0) { route = ''; }
    var middlewares = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        middlewares[_i - 1] = arguments[_i];
    }
    return Route.apply(void 0, [route, RouteMethod.Get].concat(middlewares));
}
exports.Get = Get;
/**
 * Alias function for @Route(string, RouteMethod.Put)
 *
 * @param {string} [route=''] - The routed endpoint of the method. If omitted, the base route is taken.
 * @param {RequestHandler[]} [middlewares=[]] - The used middleware(s) for this route.
 * @returns {(any, string, PropertyDescriptor) => void} - Method decorator for the given function with the http verb 'PUT'.
 */
function Put(route) {
    if (route === void 0) { route = ''; }
    var middlewares = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        middlewares[_i - 1] = arguments[_i];
    }
    return Route.apply(void 0, [route, RouteMethod.Put].concat(middlewares));
}
exports.Put = Put;
/**
 * Alias function for @Route(string, RouteMethod.Post)
 *
 * @param {string} [route=''] - The routed endpoint of the method. If omitted, the base route is taken.
 * @param {RequestHandler[]} [middlewares=[]] - The used middleware(s) for this route.
 * @returns {(any, string, PropertyDescriptor) => void} - Method decorator for the given function with the http verb 'POST'.
 */
function Post(route) {
    if (route === void 0) { route = ''; }
    var middlewares = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        middlewares[_i - 1] = arguments[_i];
    }
    return Route.apply(void 0, [route, RouteMethod.Post].concat(middlewares));
}
exports.Post = Post;
/**
 * Alias function for @Route(string, RouteMethod.Delete)
 *
 * @param {string} [route=''] - The routed endpoint of the method. If omitted, the base route is taken.
 * @param {RequestHandler[]} [middlewares=[]] - The used middleware(s) for this route.
 * @returns {(any, string, PropertyDescriptor) => void} - Method decorator for the given function with the http verb 'DELETE'.
 */
function Delete(route) {
    if (route === void 0) { route = ''; }
    var middlewares = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        middlewares[_i - 1] = arguments[_i];
    }
    return Route.apply(void 0, [route, RouteMethod.Delete].concat(middlewares));
}
exports.Delete = Delete;
/**
 * Alias function for @Route(string, RouteMethod.Head)
 *
 * @param {string} [route=''] - The routed endpoint of the method. If omitted, the base route is taken.
 * @param {RequestHandler[]} [middlewares=[]] - The used middleware(s) for this route.
 * @returns {(any, string, PropertyDescriptor) => void} - Method decorator for the given function with the http verb 'Head'.
 */
function Head(route) {
    if (route === void 0) { route = ''; }
    var middlewares = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        middlewares[_i - 1] = arguments[_i];
    }
    return Route.apply(void 0, [route, RouteMethod.Head].concat(middlewares));
}
exports.Head = Head;
