"use strict";
var Errors_1 = require('../errors/Errors');
var VersionInformation_1 = require('../models/VersionInformation');
/**
 * Reflect metadata key for the version information.
 *
 * @type {string}
 */
exports.VERSION_KEY = 'version';
/**
 * Version decorator; decorates a class or a method of a class as versioned.
 * When the routes are registered, the version information will determine
 * if a route is accessable or not. If a route is not available in a certain version
 * the system should return a 404.
 *
 * @param {{from: ?number, to: ?number}} versionInformation - An object with from and / or until information. If both are omitted, an error is thrown.
 * @returns {(Function|Object, string?, PropertyDescriptor?) => void} - Decorator for a versioned controller or route.
 */
function Version(versionInformation) {
    return function (controllerOrRoute, propertyKey, descriptor) {
        var information = VersionInformation_1.VersionInformation.create(controllerOrRoute.name, versionInformation);
        if (!propertyKey || !descriptor) {
            // Decorator is called on a class
            if (Reflect.hasMetadata(exports.VERSION_KEY, controllerOrRoute)) {
                throw new Errors_1.DuplicateVersionInformation(controllerOrRoute.name);
            }
            Reflect.defineMetadata(exports.VERSION_KEY, information, controllerOrRoute);
        }
        else {
            // Decorator is called on a method
            if (Reflect.hasMetadata(exports.VERSION_KEY, controllerOrRoute.constructor, propertyKey)) {
                throw new Errors_1.DuplicateVersionInformation(controllerOrRoute.name);
            }
            Reflect.defineMetadata(exports.VERSION_KEY, information, controllerOrRoute.constructor, propertyKey);
        }
    };
}
exports.Version = Version;
